@@ -44,6 +44,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsname = datasetname
+
         document.signalModified.connect(self.slotDocumentModified)
 
     def rowCount(self, parent):
@@ -56,10 +57,10 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
             return len(self.document.data[self.dsname].data)+1
         except (KeyError, AttributeError):
             return 0
-        
+
     def slotDocumentModified(self):
         """Called when document modified."""
-        self.emit( qt4.SIGNAL('layoutChanged()') )
+        self.layoutChanged.emit()
 
     def columnCount(self, parent):
         """Return number of columns."""
@@ -117,7 +118,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
         if index.isValid():
             f = qt4.QAbstractTableModel.flags(self, index)
             ds = self.document.data.get(self.dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
         return qt4.Qt.ItemIsEnabled
@@ -233,7 +234,7 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
 
     def slotDocumentModified(self):
         self.updateCounts()
-        self.emit( qt4.SIGNAL('layoutChanged()') )
+        self.layoutChanged.emit()
 
     def data(self, index, role):
         """Return data for index."""
@@ -276,7 +277,7 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
             f = qt4.QAbstractTableModel.flags(self, index)
             dsname, colname, dsidx, colidx = self.colattrs[index.column()]
             ds = self.document.data.get(dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
         return qt4.Qt.ItemIsEnabled
@@ -341,32 +342,50 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsname = datasetname
+        self.updatePixelCoords()
         document.signalModified.connect(self.slotDocumentModified)
 
+    def updatePixelCoords(self):
+        """Get coordinates at edge of grid."""
+        self.xedge = self.yedge = self.xcent = self.ycent = []
+        ds = self.document.data.get(self.dsname)
+        if ds:
+            self.xcent, self.ycent = ds.getPixelCentres()
+            self.xedge, self.yedge = ds.getPixelEdges()
+
     def rowCount(self, parent):
         if parent.isValid():
             return 0
-        ds = self.document.data[self.dsname].data
-        if ds is not None:
-            return ds.shape[0]
+        try:
+            data = self.document.data[self.dsname].data
+        except KeyError:
+            return 0
+        if data is not None:
+            return data.shape[0]
         else:
             return 0
 
     def columnCount(self, parent):
         if parent.isValid():
             return 0
-        ds = self.document.data[self.dsname].data
-        if ds is not None:
-            return ds.shape[1]
+        try:
+            data = self.document.data[self.dsname].data
+        except KeyError:
+            return 0
+        if data is not None:
+            return data.shape[1]
         else:
             return 0
 
     def data(self, index, role):
         if role == qt4.Qt.DisplayRole:
             # get data (note y is reversed, sigh)
-            ds = self.document.data[self.dsname].data
-            if ds is not None:
-                num = ds[ds.shape[0]-index.row()-1, index.column()]
+            try:
+                data = self.document.data[self.dsname].data
+            except KeyError:
+                return None
+            if data is not None:
+                num = data[data.shape[0]-index.row()-1, index.column()]
                 return float(num)
 
         return None
@@ -374,23 +393,26 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
     def headerData(self, section, orientation, role):
         """Return headers at top."""
 
-        if role == qt4.Qt.DisplayRole:
-            ds = self.document.data[self.dsname]
+        ds = self.document.data.get(self.dsname)
+        xaxis = orientation == qt4.Qt.Horizontal
 
-            if ds is not None:
-                # return a number for the top left of the cell
-                if orientation == qt4.Qt.Horizontal:
-                    r = ds.xrange
-                    num = ds.data.shape[1]
-                else:
-                    r = ds.yrange
-                    r = (r[1], r[0]) # swap (as y reversed)
-                    num = ds.data.shape[0]
-                val = (r[1]-r[0])/num*(section+0.5)+r[0]
-                return '%g' % val
+        # note: y coordinates are upside down (high y is at top)
+        if ds is not None and role == qt4.Qt.DisplayRole:
+            v = self.xcent[section] if xaxis else self.ycent[
+                len(self.ycent)-section-1]
+            return '%i (%s)' % (
+                len(self.ycent)-section, setting.ui_floattostring(v))
+
+        elif ds is not None and role == qt4.Qt.ToolTipRole:
+            v1 = self.xedge[section] if xaxis else self.yedge[
+                len(self.yedge)-section-2]
+            v2 = self.xedge[section+1] if xaxis else self.yedge[
+                len(self.yedge)-section-1]
+            return u'%s\u2013%s' % (setting.ui_floattostring(v1),
+                                    setting.ui_floattostring(v2))
 
         return None
-    
+
     def flags(self, index):
         """Update flags to say that items are editable."""
         if not index.isValid():
@@ -398,13 +420,14 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
         else:
             f = qt4.QAbstractTableModel.flags(self, index)
             ds = self.document.data.get(self.dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
 
     def slotDocumentModified(self):
         """Called when document modified."""
-        self.emit( qt4.SIGNAL('layoutChanged()') )
+        self.updatePixelCoords()
+        self.layoutChanged.emit()
 
     def setData(self, index, value, role):
         """Called to set the data."""
@@ -442,7 +465,7 @@ class ViewDelegate(qt4.QStyledItemDelegate):
     def setEditorData(self, editor, index):
         """Override setData to use correct formatting."""
         if type(index.data()) is float:
-            txt = setting.uilocale.toString(index.data(), 'g', 8)
+            txt = setting.ui_floattostring(index.data())
             editor.setText(txt)
         else:
             qt4.QStyledItemDelegate.setEditorData(self, editor, index)
@@ -470,7 +493,7 @@ class DataEditDialog(VeuszDialog):
             (_('Insert row'), self.slotInsertRow),
             ):
             act = qt4.QAction(text, self)
-            self.connect(act, qt4.SIGNAL('triggered()'), slot)
+            act.triggered.connect(slot)
             self.datatableview.addAction(act)
         self.datatableview.setContextMenuPolicy( qt4.Qt.ActionsContextMenu )
 
@@ -487,12 +510,12 @@ class DataEditDialog(VeuszDialog):
 
         # select first item, if any or initialise if none
         if len(self.document.data) > 0:
-            self.selectDataset( sorted(self.document.data.keys())[0] )
+            self.selectDataset( sorted(self.document.data)[0] )
         else:
             self.slotDatasetsSelected([])
 
-        self.connect(self.dsbrowser.navtree, qt4.SIGNAL("selecteddatasets"),
-                     self.slotDatasetsSelected)
+        self.dsbrowser.navtree.selecteddatasets.connect(
+            self.slotDatasetsSelected)
 
         # connect buttons
         for btn, slot in ( (self.deletebutton, self.slotDatasetDelete),
@@ -502,7 +525,7 @@ class DataEditDialog(VeuszDialog):
                            (self.createbutton, self.slotDatasetCreate),
                            (self.editbutton, self.slotDatasetEdit),
                            ):
-            self.connect(btn, qt4.SIGNAL('clicked()'), slot)
+            btn.clicked.connect(slot)
 
         # menu for new button
         self.newmenu = qt4.QMenu()
@@ -510,7 +533,7 @@ class DataEditDialog(VeuszDialog):
                             (_('Text dataset'), self.slotNewTextDataset),
                             (_('Date/time dataset'), self.slotNewDateDataset) ):
             a = self.newmenu.addAction(text)
-            self.connect(a, qt4.SIGNAL('triggered()'), slot)
+            a.triggered.connect(slot)
         self.newbutton.setMenu(self.newmenu)
 
     def slotDatasetsSelected(self, names):
