@@ -22,6 +22,7 @@
 """A class to represent Veusz documents, with dataset classes."""
 
 from __future__ import division
+import codecs
 import os.path
 import re
 import traceback
@@ -30,7 +31,12 @@ from collections import defaultdict
 
 import numpy as N
 
-from ..compat import crange, citems, cvalues, cstr, cexec
+try:
+    import h5py
+except ImportError:
+    h5py = None
+
+from ..compat import crange, citems, cvalues, cstr, cexec, CStringIO
 from .. import qtall as qt4
 
 from . import widgetfactory
@@ -75,15 +81,18 @@ def getSuitableParent(widgettype, initialwidget):
 
 class Document( qt4.QObject ):
     """Document class for holding the graph data.
-
-    Emits: sigModified when the document has been modified
-           sigWiped when document is wiped
     """
 
     pluginsloaded = False
 
     # this is emitted when the document is modified
     signalModified = qt4.pyqtSignal(int)
+    # emited to log a message
+    sigLog = qt4.pyqtSignal(cstr)
+    # emitted when document wiped
+    sigWiped = qt4.pyqtSignal()
+    # to ask whether the import is allowed (module name and symbol list)
+    sigAllowedImports = qt4.pyqtSignal(cstr, list)
 
     def __init__(self):
         """Initialise the document."""
@@ -96,10 +105,6 @@ class Document( qt4.QObject ):
         # change tracking of document as a whole
         self.changeset = 0            # increased when the document changes
 
-        # change tracking of datasets
-        self.datachangeset = 0        # increased whan any dataset changes
-        self.datachangesets = dict()  # each ds has an associated change set
-
         # map tags to dataset names
         self.datasettags = defaultdict(list)
 
@@ -130,6 +135,8 @@ class Document( qt4.QObject ):
         self.exprcompiled = {}
         self.exprfailed = set()
         self.exprfailedchangeset = -1
+        self.evalexprcache = {}
+        self.evalexprcachechangeset = None
 
     def wipe(self):
         """Wipe out any stored data."""
@@ -138,7 +145,7 @@ class Document( qt4.QObject ):
             'document', None, None)
         self.basewidget.document = self
         self.setModified(False)
-        self.emit( qt4.SIGNAL("sigWiped") )
+        self.sigWiped.emit()
 
     def clearHistory(self):
         """Clear any history."""
@@ -170,7 +177,7 @@ class Document( qt4.QObject ):
 
     def log(self, message):
         """Log a message - this is emitted as a signal."""
-        self.emit( qt4.SIGNAL("sigLog"), message )
+        self.sigLog.emit(message)
 
     def applyOperation(self, operation):
         """Apply operation to the document.
@@ -291,28 +298,18 @@ class Document( qt4.QObject ):
         dataset.document = self
         
         # update the change tracking
-        cs = self.datachangesets.get(name, 0)
-        self.datachangesets[name] = cs + 1
-        self.datachangeset += 1
         self.setModified()
     
     def deleteData(self, name):
         """Remove a dataset"""
         if name in self.data:
             del self.data[name]
-            
-            # don't remove the changeset tracker, in case this action is later undone
-            self.datachangesets[name] += 1
-            self.datachangeset += 1
             self.setModified()
 
     def modifiedData(self, dataset):
         """The named dataset was modified"""
-        for name, ds in citems(self.data):
-            if ds is dataset:
-                self.datachangesets[name] += 1
-                self.datachangeset += 1
-                self.setModified()
+        if dataset in self.data.values():
+            self.setModified()
 
     def getLinkedFiles(self, filenames=None):
         """Get a list of LinkedFile objects used by the document.
@@ -367,8 +364,6 @@ class Document( qt4.QObject ):
         d = self.data[oldname]
         del self.data[oldname]
         self.data[newname] = d
-        # transfer change set to new name
-        self.datachangesets[newname] = self.datachangesets[oldname]
 
         self.setModified()
 
@@ -472,12 +467,12 @@ class Document( qt4.QObject ):
 
         # get a list of all tags and which datasets have them
         bytag = defaultdict(list)
-        for name, dataset in sorted(citems(self.data)):
+        for name, dataset in sorted(self.data.items()):
             for t in dataset.tags:
                 bytag[t].append(name)
 
         # write out tags
-        for tag, val in sorted(citems(bytag)):
+        for tag, val in sorted(bytag.items()):
             fileobj.write('TagDatasets(%s, %s)\n' %
                           (repr(tag), repr(val)))
 
@@ -488,7 +483,16 @@ class Document( qt4.QObject ):
         self.saveCustomDefinitions(fileobj)
 
     def saveToFile(self, fileobj):
-        """Save the text representing a document to a file."""
+        """Save the text representing a document to a file.
+
+        The ordering can be important, as some things override
+        previous steps:
+
+         - Tagging doesn't work if the dataset isn't
+           already defined.
+         - Loading from files may bring in new datasets which
+           override defined datasets, so save links first
+        """
 
         self._writeFileHeader(fileobj, 'saved document')
         
@@ -504,12 +508,12 @@ class Document( qt4.QObject ):
         # save those datasets which are linked
         # we do this first in case the datasets are overridden below
         savedlinks = {}
-        for name, dataset in sorted(citems(self.data)):
+        for name, dataset in sorted(self.data.items()):
             dataset.saveLinksToSavedDoc(fileobj, savedlinks,
                                         relpath=reldirname)
 
         # save the remaining datasets
-        for name, dataset in sorted(citems(self.data)):
+        for name, dataset in sorted(self.data.items()):
             dataset.saveToFile(fileobj, name)
 
         # save tags of datasets
@@ -520,6 +524,86 @@ class Document( qt4.QObject ):
         
         self.setModified(False)
 
+    def saveToHDF5File(self, fileobj):
+        """Save to HDF5 (h5py) output file given."""
+
+        # groups in output hdf5
+        vszgrp = fileobj.create_group('Veusz')
+        vszgrp.attrs['vsz_version'] = utils.version()
+        vszgrp.attrs['vsz_saved_at'] = datetime.datetime.utcnow().isoformat()
+        vszgrp.attrs['vsz_format'] = 1  # version number (currently unused)
+        datagrp = vszgrp.create_group('Data')
+        docgrp = vszgrp.create_group('Document')
+
+        textstream = CStringIO()
+
+        self._writeFileHeader(textstream, 'saved document')
+
+        # add file directory to import path if we know it
+        reldirname = None
+        if getattr(fileobj, 'filename', False):
+            reldirname = os.path.dirname( os.path.abspath(fileobj.filename) )
+            textstream.write('AddImportPath(%s)\n' % repr(reldirname))
+
+        # add custom definitions
+        self.saveCustomDefinitions(textstream)
+
+        # save those datasets which are linked
+        # we do this first in case the datasets are overridden below
+        savedlinks = {}
+        for name, dataset in sorted(self.data.items()):
+            dataset.saveLinksToSavedDoc(textstream, savedlinks,
+                                        relpath=reldirname)
+
+        # save the remaining datasets
+        for name, dataset in sorted(self.data.items()):
+            dataset.saveToFile(textstream, name, mode='hdf5', hdfgroup=datagrp)
+
+        # handle tagging
+        # get a list of all tags and which datasets have them
+        bytag = defaultdict(list)
+        for name, dataset in sorted(self.data.items()):
+            for t in dataset.tags:
+                bytag[t].append(name)
+
+        # write out tags as datasets
+        tagsgrp = docgrp.create_group('Tags')
+        for tag, dsnames in sorted(bytag.items()):
+            tagsgrp[tag] = [v.encode('utf-8') for v in sorted(dsnames)]
+
+        # save the actual tree structure
+        textstream.write(self.basewidget.getSaveText())
+
+        # create single dataset contains document
+        docgrp['document'] = [ textstream.getvalue().encode('utf-8') ]
+
+        self.setModified(False)
+
+    def save(self, filename, mode='vsz'):
+        """Save to output file.
+
+        mode is 'vsz' or 'hdf5'
+        """
+        if mode == 'vsz':
+            with codecs.open(filename, 'w', 'utf-8') as f:
+                self.saveToFile(f)
+        elif mode == 'hdf5':
+            if h5py is None:
+                raise RuntimeError('Missing h5py module')
+            with h5py.File(filename, 'w') as f:
+                self.saveToHDF5File(f)
+        else:
+            raise RuntimeError('Invalid save mode')
+
+    def load(self, filename, mode='vsz', callbackunsafe=None):
+        """Load document from file.
+
+        mode is 'vsz' or 'hdf5'
+        """
+        from . import loader
+        loader.loadDocument(self, filename, mode=mode,
+                            callbackunsafe=callbackunsafe)
+
     def exportStyleSheet(self, fileobj):
         """Export the StyleSheet to a file."""
 
@@ -546,13 +630,18 @@ class Document( qt4.QObject ):
 
         If dpi is None, use the default Qt screen dpi
         Use dpi if given."""
-        return self._pagedocsize(self.basewidget.getPage(pagenum),
-                                 dpi=dpi, scaling=scaling, integer=integer)
+
+        page = self.basewidget.getPage(pagenum)
+        if page is None:
+            return self.docSize(dpi=dpi, scaling=scaling, integer=integer)
+        return self._pagedocsize(
+            page, dpi=dpi, scaling=scaling, integer=integer)
 
     def docSize(self, dpi=None, scaling=1., integer=True):
         """Get size for document."""
-        return self._pagedocsize(self.basewidget,
-                                 dpi=dpi, scaling=scaling, integer=integer)
+        return self._pagedocsize(
+            self.basewidget,
+            dpi=dpi, scaling=scaling, integer=integer)
 
     def resolveItem(self, fromwidget, where):
         """Resolve item relative to fromwidget.
@@ -682,8 +771,7 @@ class Document( qt4.QObject ):
 
             # only ask the user the first time
             if thepass == 0:
-                self.emit(qt4.SIGNAL('check_allowed_imports'), module,
-                          possibleimport)
+                self.sigAllowedImports.emit(module, possibleimport)
 
         return toimport
 
@@ -863,8 +951,7 @@ class Document( qt4.QObject ):
         return retn
 
     def evalDatasetExpression(self, expr, part='data', datatype='numeric',
-                              dimensions=1,
-                              _cache={}, _cache_changeset=[None]):
+                              dimensions=1):
         """Return dataset after evaluating a dataset expression.
         part is 'data', 'serr', 'perr' or 'nerr' - these are the
         dataset parts which are evaluated by the expression
@@ -873,13 +960,13 @@ class Document( qt4.QObject ):
         """
 
         key = (expr, part, datatype, dimensions)
-        if _cache_changeset[0] != self.changeset:
-            _cache_changeset[0] = self.changeset
-            _cache.clear()
-        elif key in _cache:
-            return _cache[key]
+        if self.evalexprcachechangeset != self.changeset:
+            self.evalexprcachechangeset = self.changeset
+            self.evalexprcache.clear()
+        elif key in self.evalexprcache:
+            return self.evalexprcache[key]
 
-        _cache[key] = ds = datasets.evalDatasetExpression(
+        self.evalexprcache[key] = ds = datasets.evalDatasetExpression(
             self, expr, part=part, datatype=datatype, dimensions=dimensions)
         return ds
 
@@ -936,7 +1023,7 @@ class Document( qt4.QObject ):
                            nodetypes=nodetypes, _path=_path)
         elif root.nodetype == 'settings':
             # do the settings of the settings
-            for name, s in sorted(citems(root.setdict)):
+            for name, s in sorted(root.setdict.items()):
                 self.walkNodes(tocall, root=s, nodetypes=nodetypes,
                                _path = _path + '/' + s.name)
         # elif root.nodetype == 'setting': pass
