@@ -40,12 +40,12 @@ from veusz import utils
 
 copyr='''Veusz %s
 
-Copyright (C) Jeremy Sanders 2003-2013 <jeremy@jeremysanders.net> and contributors
+Copyright (C) Jeremy Sanders 2003-2014 <jeremy@jeremysanders.net> and contributors
 Licenced under the GNU General Public Licence (version 2 or greater)
 '''
 
 splashcopyr='''<b><font color="purple">Veusz %s<br></font></b>
-Copyright (C) Jeremy Sanders 2003-2013 and contributors<br>
+Copyright (C) Jeremy Sanders 2003-2014 and contributors<br>
 Licenced under the GPL (version 2 or greater)
 '''
 
@@ -80,6 +80,8 @@ def makeSplashLogo():
 
 def excepthook(excepttype, exceptvalue, tracebackobj):
     '''Show exception dialog if an exception occurs.'''
+    sys.setrecursionlimit(sys.getrecursionlimit()+1000)
+
     from veusz.dialogs.exceptiondialog import ExceptionDialog
     if not isinstance(exceptvalue, utils.IgnoreException):
         # next exception is ignored to clear out the stack frame of the
@@ -125,6 +127,7 @@ class ImportThread(qt4.QThread):
     def run(self):
         from veusz import setting
         from veusz import widgets
+        from veusz import dataimport
 
 class VeuszApp(qt4.QApplication):
     """Event which can open mac files."""
@@ -132,8 +135,7 @@ class VeuszApp(qt4.QApplication):
     def __init__(self, args):
         qt4.QApplication.__init__(self, args)
 
-        self.connect(self, qt4.SIGNAL('lastWindowClosed()'),
-                     self, qt4.SLOT('quit()'))
+        self.lastWindowClosed.connect(self.quit)
 
         # register a signal handler to catch ctrl+C
         signal.signal(signal.SIGINT, handleIntSignal)
@@ -225,12 +227,11 @@ class VeuszApp(qt4.QApplication):
         # optionally load a translation
         if self.options.translation:
             trans = qt4.QTranslator()
-            trans.load(options.translation)
+            trans.load(self.options.translation)
             self.installTranslator(trans)
 
         self.thread = ImportThread()
-        self.connect( self.thread, qt4.SIGNAL('finished()'),
-                      self.slotStartApplication )
+        self.thread.finished.connect(self.slotStartApplication)
         self.thread.start()
 
     def slotStartApplication(self):
@@ -253,6 +254,10 @@ class VeuszApp(qt4.QApplication):
         setting.transient_settings['unsafe_mode'] = bool(
             options.unsafe_mode)
 
+        # add directories to path
+        if setting.settingdb['external_pythonpath']:
+            sys.path += setting.settingdb['external_pythonpath'].split(':')
+
         # load any requested plugins
         if options.plugin:
             document.Document.loadPlugins(pluginlist=options.plugin)
